@@ -50,6 +50,13 @@
 #define FloatToFixed(a)     ((Fixed)((float)(a) * fixed1))
 #endif
 
+/* If this isn't defined, we must be building on non-intel,
+ * hence it will be 0.
+ */
+#ifndef kCGBitmapByteOrder32Host
+#define kCGBitmapByteOrder32Host 0
+#endif
+
 typedef struct _cairo_atsui_font_face cairo_atsui_font_face_t;
 typedef struct _cairo_atsui_font cairo_atsui_font_t;
 
@@ -69,7 +76,6 @@ struct _cairo_atsui_font {
     ATSUFontID fontID;
 };
 
-
 struct _cairo_atsui_font_face {
   cairo_font_face_t base;
   ATSUFontID font_id;
@@ -109,7 +115,7 @@ static const cairo_font_face_backend_t _cairo_atsui_font_face_backend = {
     _cairo_atsui_font_face_scaled_font_create
 };
 
-cairo_public cairo_font_face_t *
+cairo_font_face_t *
 cairo_atsui_font_face_create_for_atsu_font_id (ATSUFontID font_id)
 {
   cairo_atsui_font_face_t *font_face;
@@ -127,8 +133,6 @@ cairo_atsui_font_face_create_for_atsu_font_id (ATSUFontID font_id)
     return &font_face->base;
 }
 
-
-
 static CGAffineTransform
 CGAffineTransformMakeWithCairoFontScale(const cairo_matrix_t *scale)
 {
@@ -179,7 +183,7 @@ _cairo_atsui_font_set_metrics (cairo_atsui_font_t *font)
 
         if (err == noErr) {
 	    	cairo_font_extents_t extents;
-	    
+
             extents.ascent = metrics.ascent;
             extents.descent = -metrics.descent;
             extents.height = metrics.capHeight;
@@ -320,7 +324,6 @@ _cairo_atsui_font_create_toy(cairo_toy_font_face_t *toy_face,
                             sizeof(styleTags) / sizeof(styleTags[0]),
                             styleTags, styleSizes, styleValues);
 
-
     return _cairo_atsui_font_create_scaled (&toy_face->base, fontID, style,
 					    font_matrix, ctm, options, font_out);
 }
@@ -363,9 +366,9 @@ _cairo_atsui_font_init_glyph_metrics (cairo_atsui_font_t *font,
 
    extents.x_bearing = metricsH.sideBearing.x;
    extents.y_bearing = metricsV.advance.y;
-   extents.width = 
+   extents.width =
       metricsH.advance.x - metricsH.sideBearing.x - metricsH.otherSideBearing.x;
-   extents.height = 
+   extents.height =
      -metricsV.advance.y - metricsV.sideBearing.y - metricsV.otherSideBearing.y;
    extents.x_advance = metricsH.advance.x;
    extents.y_advance = 0;
@@ -377,7 +380,7 @@ _cairo_atsui_font_init_glyph_metrics (cairo_atsui_font_t *font,
   return CAIRO_STATUS_SUCCESS;
 }
 
-static OSStatus 
+static OSStatus
 _move_to (const Float32Point *point,
 	  void *callback_data)
 {
@@ -391,7 +394,7 @@ _move_to (const Float32Point *point,
     return noErr;
 }
 
-static OSStatus 
+static OSStatus
 _line_to (const Float32Point *point,
 	  void *callback_data)
 {
@@ -400,7 +403,7 @@ _line_to (const Float32Point *point,
     _cairo_path_fixed_line_to (path,
 			       _cairo_fixed_from_double(point->x),
 			       _cairo_fixed_from_double(point->y));
-    
+
     return noErr;
 }
 
@@ -419,7 +422,7 @@ _curve_to (const Float32Point *point1,
 				_cairo_fixed_from_double(point2->y),
 				_cairo_fixed_from_double(point3->x),
 				_cairo_fixed_from_double(point3->y));
-    
+
     return noErr;
 }
 
@@ -434,7 +437,7 @@ _close_path (void *callback_data)
     return noErr;
 }
 
-static cairo_status_t 
+static cairo_status_t
 _cairo_atsui_scaled_font_init_glyph_path (cairo_atsui_font_t *scaled_font,
 					  cairo_scaled_glyph_t *scaled_glyph)
 {
@@ -468,7 +471,7 @@ _cairo_atsui_scaled_font_init_glyph_path (cairo_atsui_font_t *scaled_font,
     return CAIRO_STATUS_SUCCESS;
 }
 
-static cairo_status_t
+static cairo_int_status_t
 _cairo_atsui_font_scaled_glyph_init (void			*abstract_font,
 				     cairo_scaled_glyph_t	*scaled_glyph,
 				     cairo_scaled_glyph_info_t	 info)
@@ -491,12 +494,12 @@ _cairo_atsui_font_scaled_glyph_init (void			*abstract_font,
     return CAIRO_STATUS_SUCCESS;
 }
 
-static cairo_int_status_t 
+static cairo_int_status_t
 _cairo_atsui_font_text_to_glyphs (void		*abstract_font,
 				  double	 x,
 				  double	 y,
 				  const char	*utf8,
-				  cairo_glyph_t **glyphs, 
+				  cairo_glyph_t **glyphs,
 				  int		*num_glyphs)
 {
     cairo_status_t status = CAIRO_STATUS_SUCCESS;
@@ -528,7 +531,7 @@ _cairo_atsui_font_text_to_glyphs (void		*abstract_font,
 							&glyphCount);
 
     *num_glyphs = glyphCount - 1;
-    *glyphs = 
+    *glyphs =
 	(cairo_glyph_t *) malloc(*num_glyphs * (sizeof (cairo_glyph_t)));
     if (*glyphs == NULL) {
 	return CAIRO_STATUS_NO_MEMORY;
@@ -542,15 +545,15 @@ _cairo_atsui_font_text_to_glyphs (void		*abstract_font,
 
     free (utf16);
 
-    ATSUDirectReleaseLayoutDataArrayPtr(NULL, 
+    ATSUDirectReleaseLayoutDataArrayPtr(NULL,
 					kATSUDirectDataLayoutRecordATSLayoutRecordCurrent,
 					(void *) &layoutRecords);
     ATSUDisposeTextLayout(textLayout);
-    
+
     return CAIRO_STATUS_SUCCESS;
 }
 
-static cairo_int_status_t 
+static cairo_int_status_t
 _cairo_atsui_font_old_show_glyphs (void		       *abstract_font,
 				   cairo_operator_t    	op,
 				   cairo_pattern_t     *pattern,
@@ -571,7 +574,7 @@ _cairo_atsui_font_old_show_glyphs (void		       *abstract_font,
     int i;
     void *extra = NULL;
     cairo_bool_t can_draw_directly;
-    cairo_rectangle_fixed_t rect;
+    cairo_rectangle_int16_t rect;
 
     /* Check if we can draw directly to the destination surface */
     can_draw_directly = _cairo_surface_is_quartz (generic_surface) &&
@@ -592,7 +595,7 @@ _cairo_atsui_font_old_show_glyphs (void		       *abstract_font,
 
 	/* Create a CGBitmapContext for the dest surface for drawing into */
 	colorSpace = CGColorSpaceCreateDeviceRGB();
-	
+
 	myBitmapContext = CGBitmapContextCreate(destImageSurface->data,
 						destImageSurface->width,
 						destImageSurface->height,
@@ -651,23 +654,23 @@ _cairo_atsui_font_old_show_glyphs (void		       *abstract_font,
 		rects = malloc (sizeof (CGRect) * num_boxes);
 	    else
 		rects = stack_rects;
-	    
+
 	    for (i = 0; i < num_boxes; i++) {
 		rects[i].origin.x = boxes[i].x1;
 		rects[i].origin.y = boxes[i].y1;
 		rects[i].size.width = boxes[i].x2 - boxes[i].x1;
 		rects[i].size.height = boxes[i].y2 - boxes[i].y1;
 	    }
-	    
+
 	    CGContextClipToRects (drawingContext, rects, num_boxes);
-	    
+
 	    if (rects != stack_rects)
 		free(rects);
 	}
     } else {
 	/* XXX: Need to get the text clipped */
     }
-	
+
     /* TODO - bold and italic text
      *
      * We could draw the text using ATSUI and get bold, italics
@@ -675,10 +678,9 @@ _cairo_atsui_font_old_show_glyphs (void		       *abstract_font,
      * that we don't really need...
      */
 
-	
     for (i = 0; i < num_glyphs; i++) {
         CGGlyph theGlyph = glyphs[i].index;
-		
+
         CGContextShowGlyphsAtPoint(drawingContext,
 				   glyphs[i].x,
                                    glyphs[i].y,
@@ -688,7 +690,7 @@ _cairo_atsui_font_old_show_glyphs (void		       *abstract_font,
     if (!can_draw_directly) {
 	CGColorSpaceRelease(colorSpace);
 	CGContextRelease(myBitmapContext);
-	
+
 	_cairo_surface_release_dest_image(generic_surface,
 					  &rect,
 					  destImageSurface,
@@ -710,4 +712,3 @@ const cairo_scaled_font_backend_t cairo_atsui_scaled_font_backend = {
     NULL, /* ucs4_to_index */
     _cairo_atsui_font_old_show_glyphs,
 };
-
