@@ -44,11 +44,15 @@ _cairo_format_bpp (cairo_format_t format)
 	return 1;
     case CAIRO_FORMAT_A8:
 	return 8;
+    case CAIRO_FORMAT_RGB16_565:
+	return 16;
     case CAIRO_FORMAT_RGB24:
     case CAIRO_FORMAT_ARGB32:
-    default:
 	return 32;
     }
+
+    ASSERT_NOT_REACHED;
+    return 32;
 }
 
 cairo_surface_t *
@@ -63,7 +67,8 @@ _cairo_image_surface_create_for_pixman_image (pixman_image_t *pixman_image,
 	return (cairo_surface_t*) &_cairo_surface_nil;
     }
 
-    _cairo_surface_init (&surface->base, &cairo_image_surface_backend);
+    _cairo_surface_init (&surface->base, &cairo_image_surface_backend,
+			 _cairo_content_from_format (format));
 
     surface->pixman_image = pixman_image;
 
@@ -89,34 +94,54 @@ _cairo_format_from_pixman_format (pixman_format_t *pixman_format)
 
     pixman_format_get_masks (pixman_format, &bpp, &am, &rm, &gm, &bm);
 
-    if (bpp == 32 &&
-	am == 0xff000000 &&
-	rm == 0x00ff0000 &&
-	gm == 0x0000ff00 &&
-	bm == 0x000000ff)
-	return CAIRO_FORMAT_ARGB32;
+    switch (bpp) {
+    case 32:
+	if (am == 0xff000000 &&
+	    rm == 0x00ff0000 &&
+	    gm == 0x0000ff00 &&
+	    bm == 0x000000ff)
+	    return CAIRO_FORMAT_ARGB32;
+	if (am == 0x0 &&
+	    rm == 0x00ff0000 &&
+	    gm == 0x0000ff00 &&
+	    bm == 0x000000ff)
+	    return CAIRO_FORMAT_RGB24;
+	break;
+    case 16:
+	if (am == 0x0 &&
+	    rm == 0xf800 &&
+	    gm == 0x07e0 &&
+	    bm == 0x001f)
+	    return CAIRO_FORMAT_RGB16_565;
+	break;
+    case 8:
+	if (am == 0xff &&
+	    rm == 0x0 &&
+	    gm == 0x0 &&
+	    bm == 0x0)
+	    return CAIRO_FORMAT_A8;
+	break;
+    case 1:
+	if (am == 0x1 &&
+	    rm == 0x0 &&
+	    gm == 0x0 &&
+	    bm == 0x0)
+	    return CAIRO_FORMAT_A1;
+	break;
+    }
 
-    if (bpp == 32 &&
-	am == 0x0 &&
-	rm == 0x00ff0000 &&
-	gm == 0x0000ff00 &&
-	bm == 0x000000ff)
-	return CAIRO_FORMAT_RGB24;
-
-    if (bpp == 8 &&
-	am == 0xff &&
-	rm == 0x0 &&
-	gm == 0x0 &&
-	bm == 0x0)
-	return CAIRO_FORMAT_A8;
-
-    if (bpp == 1 &&
-	am == 0x1 &&
-	rm == 0x0 &&
-	gm == 0x0 &&
-	bm == 0x0)
-	return CAIRO_FORMAT_A1;
+    fprintf (stderr,
+	     "Error: Cairo does not yet support the requested image format:\n"
+	     "\tDepth: %d\n"
+	     "\tAlpha mask: 0x%08x\n"
+	     "\tRed   mask: 0x%08x\n"
+	     "\tBlue  mask: 0x%08x\n"
+	     "\tGreen mask: 0x%08x\n"
+	     "Please file an enhacement request (quoting the above) at:\n"
+	     PACKAGE_BUGREPORT "\n",
+	     bpp, am, rm, gm, bm);
 
+    ASSERT_NOT_REACHED;
     return (cairo_format_t) -1;
 }
 
@@ -171,6 +196,9 @@ _create_pixman_format (cairo_format_t format)
     case CAIRO_FORMAT_A8:
 	return pixman_format_create (PIXMAN_FORMAT_NAME_A8);
 	break;
+    case CAIRO_FORMAT_RGB16_565:
+	return pixman_format_create (PIXMAN_FORMAT_NAME_RGB16_565);
+	break;
     case CAIRO_FORMAT_RGB24:
 	return pixman_format_create (PIXMAN_FORMAT_NAME_RGB24);
 	break;
@@ -183,10 +211,10 @@ _create_pixman_format (cairo_format_t format)
 
 /**
  * cairo_image_surface_create:
- * @format: format of pixels in the surface to create 
+ * @format: format of pixels in the surface to create
  * @width: width of the surface, in pixels
  * @height: height of the surface, in pixels
- * 
+ *
  * Creates an image surface of the specified format and
  * dimensions. Initially the surface contents are all
  * 0. (Specifically, within each pixel, each color or alpha channel
@@ -256,7 +284,7 @@ _cairo_image_surface_create_with_content (cairo_content_t	content,
  *   in the buffer. Having this be specified separate from @width
  *   allows for padding at the end of rows, or for writing
  *   to a subportion of a larger image.
- * 
+ *
  * Creates an image surface for the provided pixel data. The output
  * buffer must be kept around until the #cairo_surface_t is destroyed
  * or cairo_surface_finish() is called on the surface.  The initial
@@ -294,7 +322,7 @@ cairo_image_surface_create_for_data (unsigned char     *data,
 	_cairo_error (CAIRO_STATUS_NO_MEMORY);
 	return (cairo_surface_t*) &_cairo_surface_nil;
     }
-    
+
     pixman_image = pixman_image_create_for_data ((pixman_bits_t *) data, pixman_format,
 						 width, height,
 						 _cairo_format_bpp (format),
@@ -327,12 +355,60 @@ _cairo_image_surface_create_for_data_with_content (unsigned char	*data,
 						width, height, stride);
 }
 
+/**
+ * cairo_image_surface_get_data:
+ * @surface: a #cairo_image_surface_t
+ *
+ * Get a pointer to the data of the image surface, for direct
+ * inspection or modification.
+ *
+ * Return value: a pointer to the image data of this surface or NULL
+ * if @surface is not an image surface.
+ *
+ * Since: 1.2
+ **/
+unsigned char *
+cairo_image_surface_get_data (cairo_surface_t *surface)
+{
+    cairo_image_surface_t *image_surface = (cairo_image_surface_t *) surface;
+
+    if (!_cairo_surface_is_image (surface)) {
+	_cairo_error (CAIRO_STATUS_SURFACE_TYPE_MISMATCH);
+	return NULL;
+    }
+
+    return image_surface->data;
+}
+
+/**
+ * cairo_image_surface_get_format:
+ * @surface: a #cairo_image_surface_t
+ *
+ * Get the format of the surface.
+ *
+ * Return value: the format of the surface
+ *
+ * Since: 1.2
+ **/
+cairo_format_t
+cairo_image_surface_get_format (cairo_surface_t *surface)
+{
+    cairo_image_surface_t *image_surface = (cairo_image_surface_t *) surface;
+
+    if (!_cairo_surface_is_image (surface)) {
+	_cairo_error (CAIRO_STATUS_SURFACE_TYPE_MISMATCH);
+	return 0;
+    }
+
+    return image_surface->format;
+}
+
 /**
  * cairo_image_surface_get_width:
  * @surface: a #cairo_image_surface_t
- * 
+ *
  * Get the width of the image surface in pixels.
- * 
+ *
  * Return value: the width of the surface in pixels.
  **/
 int
@@ -351,9 +427,9 @@ cairo_image_surface_get_width (cairo_surface_t *surface)
 /**
  * cairo_image_surface_get_height:
  * @surface: a #cairo_image_surface_t
- * 
+ *
  * Get the height of the image surface in pixels.
- * 
+ *
  * Return value: the height of the surface in pixels.
  **/
 int
@@ -369,6 +445,33 @@ cairo_image_surface_get_height (cairo_surface_t *surface)
     return image_surface->height;
 }
 
+/**
+ * cairo_image_surface_get_stride:
+ * @surface: a #cairo_image_surface_t
+ *
+ * Get the stride of the image surface in bytes
+ *
+ * Return value: the stride of the image surface in bytes (or 0 if
+ * @surface is not an image surface). The stride is the distance in
+ * bytes from the beginning of one row of the image data to the
+ * beginning of the next row.
+ *
+ * Since: 1.2
+ **/
+int
+cairo_image_surface_get_stride (cairo_surface_t *surface)
+{
+
+    cairo_image_surface_t *image_surface = (cairo_image_surface_t *) surface;
+
+    if (!_cairo_surface_is_image (surface)) {
+	_cairo_error (CAIRO_STATUS_SURFACE_TYPE_MISMATCH);
+	return 0;
+    }
+
+    return image_surface->stride;
+}
+
 cairo_format_t
 _cairo_format_from_content (cairo_content_t content)
 {
@@ -392,6 +495,7 @@ _cairo_content_from_format (cairo_format_t format)
     case CAIRO_FORMAT_ARGB32:
 	return CAIRO_CONTENT_COLOR_ALPHA;
     case CAIRO_FORMAT_RGB24:
+    case CAIRO_FORMAT_RGB16_565:
 	return CAIRO_CONTENT_COLOR;
     case CAIRO_FORMAT_A8:
     case CAIRO_FORMAT_A1:
@@ -445,7 +549,7 @@ _cairo_image_surface_acquire_source_image (void                    *abstract_sur
 {
     *image_out = abstract_surface;
     *image_extra = NULL;
-    
+
     return CAIRO_STATUS_SUCCESS;
 }
 
@@ -458,13 +562,13 @@ _cairo_image_surface_release_source_image (void                   *abstract_surf
 
 static cairo_status_t
 _cairo_image_surface_acquire_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_fixed_t *interest_rect,
+					 cairo_rectangle_int16_t *interest_rect,
 					 cairo_image_surface_t  **image_out,
-					 cairo_rectangle_fixed_t *image_rect_out,
+					 cairo_rectangle_int16_t *image_rect_out,
 					 void                   **image_extra)
 {
     cairo_image_surface_t *surface = abstract_surface;
-    
+
     image_rect_out->x = 0;
     image_rect_out->y = 0;
     image_rect_out->width = surface->width;
@@ -472,15 +576,15 @@ _cairo_image_surface_acquire_dest_image (void                    *abstract_surfa
 
     *image_out = surface;
     *image_extra = NULL;
-    
+
     return CAIRO_STATUS_SUCCESS;
 }
 
 static void
 _cairo_image_surface_release_dest_image (void                    *abstract_surface,
-					 cairo_rectangle_fixed_t *interest_rect,
+					 cairo_rectangle_int16_t *interest_rect,
 					 cairo_image_surface_t   *image,
-					 cairo_rectangle_fixed_t *image_rect,
+					 cairo_rectangle_int16_t *image_rect,
 					 void                    *image_extra)
 {
 }
@@ -496,8 +600,8 @@ _cairo_image_surface_clone_similar (void		*abstract_surface,
 	*clone_out = cairo_surface_reference (src);
 
 	return CAIRO_STATUS_SUCCESS;
-    }	
-    
+    }
+
     return CAIRO_INT_STATUS_UNSUPPORTED;
 }
 
@@ -549,7 +653,7 @@ _cairo_image_surface_set_attributes (cairo_image_surface_t      *surface,
 				     cairo_surface_attributes_t *attributes)
 {
     cairo_int_status_t status;
-    
+
     status = _cairo_image_surface_set_matrix (surface, &attributes->matrix);
     if (status)
 	return status;
@@ -568,9 +672,9 @@ _cairo_image_surface_set_attributes (cairo_image_surface_t      *surface,
         pixman_image_set_repeat (surface->pixman_image, PIXMAN_REPEAT_PAD);
 	break;
     }
-    
+
     status = _cairo_image_surface_set_filter (surface, attributes->filter);
-    
+
     return status;
 }
 
@@ -649,7 +753,7 @@ _cairo_image_surface_composite (cairo_operator_t	op,
 					      &src_attr, &mask_attr);
     if (status)
 	return status;
-    
+
     status = _cairo_image_surface_set_attributes (src, &src_attr);
     if (status)
       goto CLEANUP_SURFACES;
@@ -659,7 +763,7 @@ _cairo_image_surface_composite (cairo_operator_t	op,
 	status = _cairo_image_surface_set_attributes (mask, &mask_attr);
 	if (status)
 	    goto CLEANUP_SURFACES;
-	
+
 	pixman_composite (_pixman_operator (op),
 			  src->pixman_image,
 			  mask->pixman_image,
@@ -683,7 +787,7 @@ _cairo_image_surface_composite (cairo_operator_t	op,
 			  dst_x, dst_y,
 			  width, height);
     }
-    
+
     if (!_cairo_operator_bounded_by_source (op))
 	status = _cairo_surface_composite_fixup_unbounded (&dst->base,
 							   &src_attr, src->width, src->height,
@@ -697,9 +801,9 @@ _cairo_image_surface_composite (cairo_operator_t	op,
  CLEANUP_SURFACES:
     if (mask)
 	_cairo_pattern_release_surface (mask_pattern, &mask->base, &mask_attr);
-    
+
     _cairo_pattern_release_surface (src_pattern, &src->base, &src_attr);
-    
+
     return status;
 }
 
@@ -707,7 +811,7 @@ static cairo_int_status_t
 _cairo_image_surface_fill_rectangles (void		      *abstract_surface,
 				      cairo_operator_t	       op,
 				      const cairo_color_t     *color,
-				      cairo_rectangle_fixed_t *rects,
+				      cairo_rectangle_int16_t *rects,
 				      int		       num_rects)
 {
     cairo_image_surface_t *surface = abstract_surface;
@@ -726,20 +830,6 @@ _cairo_image_surface_fill_rectangles (void		      *abstract_surface,
     return CAIRO_STATUS_SUCCESS;
 }
 
-static cairo_bool_t
-_cairo_image_surface_is_alpha_only (cairo_image_surface_t *surface)
-{
-    int bpp, alpha, red, green, blue;
-    
-    if (surface->format != (cairo_format_t) -1)
-	return surface->format == CAIRO_FORMAT_A1 || surface->format == CAIRO_FORMAT_A8;
-
-    pixman_format_get_masks (pixman_image_get_format (surface->pixman_image),
-			     &bpp, &alpha, &red, &green, &blue);
-
-    return red == 0 && blue == 0 && green == 0;
-}
-
 static cairo_int_status_t
 _cairo_image_surface_composite_trapezoids (cairo_operator_t	op,
 					   cairo_pattern_t	*pattern,
@@ -779,7 +869,7 @@ _cairo_image_surface_composite_trapezoids (cairo_operator_t	op,
      */
     if (op == CAIRO_OPERATOR_ADD &&
 	_cairo_pattern_is_opaque_solid (pattern) &&
-	_cairo_image_surface_is_alpha_only (dst) &&
+	dst->base.content == CAIRO_CONTENT_ALPHA &&
 	!dst->has_clip &&
 	antialias != CAIRO_ANTIALIAS_NONE)
     {
@@ -880,7 +970,7 @@ _cairo_image_surface_set_clip_region (void *abstract_surface,
 
 static cairo_int_status_t
 _cairo_image_surface_get_extents (void			  *abstract_surface,
-				  cairo_rectangle_fixed_t *rectangle)
+				  cairo_rectangle_int16_t *rectangle)
 {
     cairo_image_surface_t *surface = abstract_surface;
 
@@ -895,9 +985,9 @@ _cairo_image_surface_get_extents (void			  *abstract_surface,
 /**
  * _cairo_surface_is_image:
  * @surface: a #cairo_surface_t
- * 
+ *
  * Checks if a surface is an #cairo_image_surface_t
- * 
+ *
  * Return value: TRUE if the surface is an image surface
  **/
 cairo_bool_t
