@@ -50,6 +50,8 @@ typedef struct cairo_stroker {
     cairo_point_t current_point;
     cairo_point_t first_point;
 
+    cairo_bool_t has_sub_path;
+
     cairo_bool_t has_current_face;
     cairo_stroke_face_t current_face;
 
@@ -161,9 +163,10 @@ _cairo_stroker_init (cairo_stroker_t		*stroker,
     _cairo_pen_init (&stroker->pen,
 		     stroke_style->line_width / 2.0,
 		     tolerance, ctm);
-    
+
     stroker->has_current_face = FALSE;
     stroker->has_first_face = FALSE;
+    stroker->has_sub_path = FALSE;
 
     if (stroker->style->dash)
 	_cairo_stroker_start_dash (stroker);
@@ -205,7 +208,8 @@ _cairo_stroker_join (cairo_stroker_t *stroker, cairo_stroke_face_t *in, cairo_st
     if (in->cw.x == out->cw.x
 	&& in->cw.y == out->cw.y
 	&& in->ccw.x == out->ccw.x
-	&& in->ccw.y == out->ccw.y) {
+	&& in->ccw.y == out->ccw.y)
+    {
 	return CAIRO_STATUS_SUCCESS;
     }
 
@@ -286,7 +290,7 @@ _cairo_stroker_join (cairo_stroker_t *stroker, cairo_stroke_face_t *in, cairo_st
 	 *				in · out = cos (psi)
 	 *
 	 *	2 <= ml² (1 - in · out)
-	 * 	 
+	 *
 	 */
 	if (2 <= ml * ml * (1 - in_dot_out)) {
 	    double		x1, y1, x2, y2;
@@ -295,7 +299,7 @@ _cairo_stroker_join (cairo_stroker_t *stroker, cairo_stroke_face_t *in, cairo_st
 	    cairo_polygon_t	polygon;
 	    cairo_point_t	outer;
 
-	    /* 
+	    /*
 	     * we've got the points already transformed to device
 	     * space, but need to do some computation with them and
 	     * also need to transform the slope from user space to
@@ -307,14 +311,14 @@ _cairo_stroker_join (cairo_stroker_t *stroker, cairo_stroke_face_t *in, cairo_st
 	    dx1 = in->usr_vector.x;
 	    dy1 = in->usr_vector.y;
 	    cairo_matrix_transform_distance (stroker->ctm, &dx1, &dy1);
-	    
+
 	    /* outer point of outgoing line face */
 	    x2 = _cairo_fixed_to_double (outpt->x);
 	    y2 = _cairo_fixed_to_double (outpt->y);
 	    dx2 = out->usr_vector.x;
 	    dy2 = out->usr_vector.y;
 	    cairo_matrix_transform_distance (stroker->ctm, &dx2, &dy2);
-	    
+
 	    /*
 	     * Compute the location of the outer corner of the miter.
 	     * That's pretty easy -- just the intersection of the two
@@ -329,7 +333,7 @@ _cairo_stroker_join (cairo_stroker_t *stroker, cairo_stroke_face_t *in, cairo_st
 		mx = (my - y1) * dx1 / dy1 + x1;
 	    else
 		mx = (my - y2) * dx2 / dy2 + x2;
-	    
+
 	    /*
 	     * Draw the quadrilateral
 	     */
@@ -368,7 +372,7 @@ _cairo_stroker_add_cap (cairo_stroker_t *stroker, cairo_stroke_face_t *f)
 
     if (stroker->style->line_cap == CAIRO_LINE_CAP_BUTT)
 	return CAIRO_STATUS_SUCCESS;
-    
+
     switch (stroker->style->line_cap) {
     case CAIRO_LINE_CAP_ROUND: {
 	int i;
@@ -459,10 +463,26 @@ _cairo_stroker_add_trailing_cap (cairo_stroker_t     *stroker,
     return _cairo_stroker_add_cap (stroker, face);
 }
 
+static void
+_compute_face (cairo_point_t *point, cairo_slope_t *slope, cairo_stroker_t *stroker, cairo_stroke_face_t *face);
+
 static cairo_status_t
 _cairo_stroker_add_caps (cairo_stroker_t *stroker)
 {
     cairo_status_t status;
+    /* check for a degenerative sub_path */
+    if (stroker->has_sub_path
+	&& !stroker->has_first_face
+	&& !stroker->has_current_face
+	&& stroker->style->line_cap == CAIRO_LINE_JOIN_ROUND)
+    {
+	/* pick an arbitrary slope to use */
+	cairo_slope_t slope = {1, 0};
+	_compute_face (&stroker->first_point, &slope, stroker, &stroker->first_face);
+
+	stroker->has_first_face = stroker->has_current_face = TRUE;
+	stroker->current_face = stroker->first_face;
+    }
 
     if (stroker->has_first_face) {
 	status = _cairo_stroker_add_leading_cap (stroker, &stroker->first_face);
@@ -507,7 +527,7 @@ _compute_face (cairo_point_t *point, cairo_slope_t *slope, cairo_stroker_t *stro
     usr_vector.x = line_dx;
     usr_vector.y = line_dy;
 
-    /* 
+    /*
      * rotate to get a line_width/2 vector along the face, note that
      * the vector must be rotated the right direction in device space,
      * but by 90° in user space. So, the rotation depends on
@@ -563,12 +583,8 @@ _cairo_stroker_add_sub_edge (cairo_stroker_t *stroker, cairo_point_t *p1, cairo_
        fields from start. */
     _compute_face (p2, slope, stroker, end);
 
-    if (p1->x == p2->x && p1->y == p2->y) {
-	/* XXX: Need to rethink how this case should be handled, (both
-           here and in _compute_face). The key behavior is that
-           degenerate paths should draw as much as possible. */
+    if (p1->x == p2->x && p1->y == p2->y)
 	return CAIRO_STATUS_SUCCESS;
-    }
 
     /* XXX: I should really check the return value of the
        move_to/line_to functions here to catch out of memory
@@ -609,8 +625,9 @@ _cairo_stroker_move_to (void *closure, cairo_point_t *point)
     stroker->first_point = *point;
     stroker->current_point = *point;
 
-    stroker->has_first_face = 0;
-    stroker->has_current_face = 0;
+    stroker->has_first_face = FALSE;
+    stroker->has_current_face = FALSE;
+    stroker->has_sub_path = FALSE;
 
     return CAIRO_STATUS_SUCCESS;
 }
@@ -635,16 +652,13 @@ _cairo_stroker_line_to (void *closure, cairo_point_t *point)
     cairo_point_t *p2 = point;
     cairo_slope_t slope;
 
-    if (p1->x == p2->x && p1->y == p2->y) {
-	/* XXX: Need to rethink how this case should be handled, (both
-           here and in cairo_stroker_add_sub_edge and in _compute_face). The
-           key behavior is that degenerate paths should draw as much
-           as possible. */
+    stroker->has_sub_path = TRUE;
+
+    if (p1->x == p2->x && p1->y == p2->y)
 	return CAIRO_STATUS_SUCCESS;
-    }
 
     _cairo_slope_init (&slope, p1, p2);
-    
+
     status = _cairo_stroker_add_sub_edge (stroker, p1, p2, &slope, &start, &end);
     if (status)
 	return status;
@@ -656,11 +670,11 @@ _cairo_stroker_line_to (void *closure, cairo_point_t *point)
     } else {
 	if (!stroker->has_first_face) {
 	    stroker->first_face = start;
-	    stroker->has_first_face = 1;
+	    stroker->has_first_face = TRUE;
 	}
     }
     stroker->current_face = end;
-    stroker->has_current_face = 1;
+    stroker->has_current_face = TRUE;
 
     stroker->current_point = *point;
 
@@ -679,19 +693,14 @@ _cairo_stroker_line_to_dashed (void *closure, cairo_point_t *point)
     double dx, dy;
     double dx2, dy2;
     cairo_point_t fd1, fd2;
-    int first = 1;
+    cairo_bool_t first = TRUE;
     cairo_stroke_face_t sub_start, sub_end;
     cairo_point_t *p1 = &stroker->current_point;
     cairo_point_t *p2 = point;
     cairo_slope_t slope;
 
-    if (p1->x == p2->x && p1->y == p2->y) {
-	/* XXX: Need to rethink how this case should be handled, (both
-           here and in cairo_stroker_add_sub_edge and in _compute_face). The
-           key behavior is that degenerate paths should draw as much
-           as possible. */
+    if (p1->x == p2->x && p1->y == p2->y)
 	return CAIRO_STATUS_SUCCESS;
-    }
 
     _cairo_slope_init (&slope, p1, p2);
 
@@ -742,7 +751,7 @@ _cairo_stroker_line_to_dashed (void *closure, cairo_point_t *point)
 		} else {
 		    if (!stroker->has_first_face) {
 			stroker->first_face = sub_start;
-			stroker->has_first_face = 1;
+			stroker->has_first_face = TRUE;
 		    } else {
 			status = _cairo_stroker_add_leading_cap (stroker, &sub_start);
 			if (status)
@@ -763,7 +772,7 @@ _cairo_stroker_line_to_dashed (void *closure, cairo_point_t *point)
 		 * through
 		 */
 		stroker->current_face = sub_end;
-		stroker->has_current_face = 1;
+		stroker->has_current_face = TRUE;
 	    }
 	} else {
 	    /*
@@ -778,11 +787,11 @@ _cairo_stroker_line_to_dashed (void *closure, cairo_point_t *point)
 		}
 	    }
 	    if (!remain)
-		stroker->has_current_face = 0;
+		stroker->has_current_face = FALSE;
 	}
 	_cairo_stroker_step_dash (stroker, tmp);
 	fd1 = fd2;
-	first = 0;
+	first = FALSE;
     }
 
     stroker->current_point = *point;
@@ -822,12 +831,12 @@ _cairo_stroker_curve_to (void *closure,
     } else {
 	if (!stroker->has_first_face) {
 	    stroker->first_face = start;
-	    stroker->has_first_face = 1;
+	    stroker->has_first_face = TRUE;
 	}
     }
     stroker->current_face = end;
-    stroker->has_current_face = 1;
-    
+    stroker->has_current_face = TRUE;
+
     extra_points[0] = start.cw;
     extra_points[0].x -= start.point.x;
     extra_points[0].y -= start.point.y;
@@ -840,7 +849,7 @@ _cairo_stroker_curve_to (void *closure,
     extra_points[3] = end.ccw;
     extra_points[3].x -= end.point.x;
     extra_points[3].y -= end.point.y;
-    
+
     status = _cairo_pen_add_points (&pen, extra_points, 4);
     if (status)
 	goto CLEANUP_PEN;
@@ -943,10 +952,15 @@ _cairo_stroker_close_path (void *closure)
 	status = _cairo_stroker_join (stroker, &stroker->current_face, &stroker->first_face);
 	if (status)
 	    return status;
+    } else {
+	status = _cairo_stroker_add_caps (stroker);
+	if (status)
+	    return status;
     }
 
-    stroker->has_first_face = 0;
-    stroker->has_current_face = 0;
+    stroker->has_sub_path = FALSE;
+    stroker->has_first_face = FALSE;
+    stroker->has_current_face = FALSE;
 
     return CAIRO_STATUS_SUCCESS;
 }
