@@ -37,7 +37,7 @@
 #include "cairoint.h"
 #include "cairo-win32-private.h"
 
-#ifndef SPI_GETFONTSMOOTHINGTYPE 
+#ifndef SPI_GETFONTSMOOTHINGTYPE
 #define SPI_GETFONTSMOOTHINGTYPE 0x200a
 #endif
 #ifndef FE_FONTSMOOTHINGCLEARTYPE
@@ -52,8 +52,6 @@
 
 const cairo_scaled_font_backend_t cairo_win32_scaled_font_backend;
 
-#define LOGICAL_SCALE 32
-
 typedef struct {
     cairo_scaled_font_t base;
 
@@ -63,8 +61,8 @@ typedef struct {
 
     /* We do drawing and metrics computation in a "logical space" which
      * is similar to font space, except that it is scaled by a factor
-     * of the (desired font size) * (LOGICAL_SCALE). The multiplication
-     * by LOGICAL_SCALE allows for sub-pixel precision.
+     * of the (desired font size) * (WIN32_FONT_LOGICAL_SCALE). The multiplication
+     * by WIN32_FONT_LOGICAL_SCALE allows for sub-pixel precision.
      */
     double logical_scale;
 
@@ -91,20 +89,21 @@ typedef struct {
     cairo_bool_t swap_y;
     double x_scale;
     double y_scale;
-  
+
     /* The size of the design unit of the font
      */
     int em_square;
 
     HFONT scaled_hfont;
     HFONT unscaled_hfont;
-    
+
+    cairo_bool_t delete_scaled_hfont;
 } cairo_win32_scaled_font_t;
 
-static cairo_status_t 
+static cairo_status_t
 _cairo_win32_scaled_font_set_metrics (cairo_win32_scaled_font_t *scaled_font);
 
-static cairo_status_t 
+static cairo_status_t
 _cairo_win32_scaled_font_init_glyph_metrics (cairo_win32_scaled_font_t *scaled_font,
 					     cairo_scaled_glyph_t      *scaled_glyph);
 
@@ -127,7 +126,7 @@ _compute_transform (cairo_win32_scaled_font_t *scaled_font,
 	scaled_font->y_scale = sc->yy;
 	scaled_font->swap_y = (sc->yy < 0);
 	scaled_font->swap_axes = FALSE;
-	
+
     } else if (NEARLY_ZERO (sc->xx) && NEARLY_ZERO (sc->yy)) {
 	scaled_font->preserve_axes = TRUE;
 	scaled_font->x_scale = sc->yx;
@@ -146,9 +145,9 @@ _compute_transform (cairo_win32_scaled_font_t *scaled_font,
 	    scaled_font->x_scale = - scaled_font->x_scale;
 	if (scaled_font->swap_y)
 	    scaled_font->y_scale = - scaled_font->y_scale;
-	
-	scaled_font->logical_scale = LOGICAL_SCALE * scaled_font->y_scale;
-	scaled_font->logical_size = LOGICAL_SCALE * floor (scaled_font->y_scale + 0.5);
+
+	scaled_font->logical_scale = WIN32_FONT_LOGICAL_SCALE * scaled_font->y_scale;
+	scaled_font->logical_size = WIN32_FONT_LOGICAL_SCALE * floor (scaled_font->y_scale + 0.5);
     }
 
     /* The font matrix has x and y "scale" components which we extract and
@@ -162,15 +161,15 @@ _compute_transform (cairo_win32_scaled_font_t *scaled_font,
 					     &scaled_font->x_scale, &scaled_font->y_scale,
 					     TRUE);	/* XXX: Handle vertical text */
 
-	scaled_font->logical_size = floor (LOGICAL_SCALE * scaled_font->y_scale + 0.5);
-	scaled_font->logical_scale = LOGICAL_SCALE * scaled_font->y_scale;
+	scaled_font->logical_size = floor (WIN32_FONT_LOGICAL_SCALE * scaled_font->y_scale + 0.5);
+	scaled_font->logical_scale = WIN32_FONT_LOGICAL_SCALE * scaled_font->y_scale;
     }
 
     cairo_matrix_scale (&scaled_font->logical_to_device,
 			1.0 / scaled_font->logical_scale, 1.0 / scaled_font->logical_scale);
 
     scaled_font->device_to_logical = scaled_font->logical_to_device;
-    
+
     status = cairo_matrix_invert (&scaled_font->device_to_logical);
     if (status)
 	cairo_matrix_init_identity (&scaled_font->device_to_logical);
@@ -180,20 +179,19 @@ static cairo_bool_t
 _have_cleartype_quality (void)
 {
     OSVERSIONINFO version_info;
-    
+
     version_info.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
-    
+
     if (!GetVersionEx (&version_info)) {
 	_cairo_win32_print_gdi_error ("_have_cleartype_quality");
 	return FALSE;
     }
-    
+
     return (version_info.dwMajorVersion > 5 ||
 	    (version_info.dwMajorVersion == 5 &&
 	     version_info.dwMinorVersion >= 1));	/* XP or newer */
 }
 
-
 static BYTE
 _get_system_quality (void)
 {
@@ -225,10 +223,11 @@ _get_system_quality (void)
 
 static cairo_scaled_font_t *
 _win32_scaled_font_create (LOGFONTW                   *logfont,
+			   HFONT                      hfont,
 			   cairo_font_face_t	      *font_face,
 			   const cairo_matrix_t       *font_matrix,
 			   const cairo_matrix_t       *ctm,
-			   const cairo_font_options_t *options)			  
+			   const cairo_font_options_t *options)
 {
     cairo_win32_scaled_font_t *f;
     cairo_matrix_t scale;
@@ -244,7 +243,7 @@ _win32_scaled_font_create (LOGFONTW                   *logfont,
      * order in the Win32 font API, so we ignore those parts of
      * cairo_font_options_t. We use the 'antialias' field to set
      * the 'quality'.
-     * 
+     *
      * XXX: The other option we could pay attention to, but don't
      *      here is the hint_metrics options.
      */
@@ -268,11 +267,14 @@ _win32_scaled_font_create (LOGFONTW                   *logfont,
 	    ASSERT_NOT_REACHED;
 	}
     }
-    
+
     f->em_square = 0;
-    f->scaled_hfont = NULL;
+    f->scaled_hfont = hfont;
     f->unscaled_hfont = NULL;
 
+    /* don't delete the hfont if it was passed in to us */
+    f->delete_scaled_hfont = !hfont;
+
     cairo_matrix_multiply (&scale, font_matrix, ctm);
     _compute_transform (f, &scale);
 
@@ -294,7 +296,7 @@ _win32_scaled_font_set_world_transform (cairo_win32_scaled_font_t *scaled_font,
 					HDC                        hdc)
 {
     XFORM xform;
-    
+
     xform.eM11 = scaled_font->logical_to_device.xx;
     xform.eM21 = scaled_font->logical_to_device.xy;
     xform.eM12 = scaled_font->logical_to_device.yx;
@@ -384,7 +386,7 @@ _win32_scaled_font_get_unscaled_hfont (cairo_win32_scaled_font_t *scaled_font,
 	    _cairo_win32_print_gdi_error ("_win32_scaled_font_get_unscaled_hfont:GetOutlineTextMetrics");
 	    return NULL;
 	}
-	
+
 	otm = malloc (otm_size);
 	if (!otm)
 	    return NULL;
@@ -397,14 +399,14 @@ _win32_scaled_font_get_unscaled_hfont (cairo_win32_scaled_font_t *scaled_font,
 
 	scaled_font->em_square = otm->otmEMSquare;
 	free (otm);
-	
+
 	logfont = scaled_font->logfont;
 	logfont.lfHeight = -scaled_font->em_square;
 	logfont.lfWidth = 0;
 	logfont.lfEscapement = 0;
 	logfont.lfOrientation = 0;
 	logfont.lfQuality = scaled_font->quality;
-	
+
 	scaled_font->unscaled_hfont = CreateFontIndirectW (&logfont);
 	if (!scaled_font->unscaled_hfont) {
 	    _cairo_win32_print_gdi_error ("_win32_scaled_font_get_unscaled_hfont:CreateIndirect");
@@ -507,7 +509,7 @@ _cairo_win32_scaled_font_create_toy (cairo_toy_font_face_t *toy_face,
      * interpretation is locale-specific, but it's not clear what
      * would be a better alternative.
      */
-    logfont.lfCharSet = DEFAULT_CHARSET; 
+    logfont.lfCharSet = DEFAULT_CHARSET;
     logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
     logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
     logfont.lfQuality = DEFAULT_QUALITY; /* filled in later */
@@ -515,8 +517,8 @@ _cairo_win32_scaled_font_create_toy (cairo_toy_font_face_t *toy_face,
 
     if (!logfont.lfFaceName)
 	return CAIRO_STATUS_NO_MEMORY;
-    
-    scaled_font = _win32_scaled_font_create (&logfont, &toy_face->base,
+
+    scaled_font = _win32_scaled_font_create (&logfont, NULL, &toy_face->base,
 					     font_matrix, ctm, options);
     if (!scaled_font)
 	return CAIRO_STATUS_NO_MEMORY;
@@ -526,7 +528,7 @@ _cairo_win32_scaled_font_create_toy (cairo_toy_font_face_t *toy_face,
     return CAIRO_STATUS_SUCCESS;
 }
 
-static void 
+static void
 _cairo_win32_scaled_font_fini (void *abstract_font)
 {
     cairo_win32_scaled_font_t *scaled_font = abstract_font;
@@ -534,19 +536,19 @@ _cairo_win32_scaled_font_fini (void *abstract_font)
     if (scaled_font == NULL)
 	return;
 
-    if (scaled_font->scaled_hfont)
+    if (scaled_font->scaled_hfont && scaled_font->delete_scaled_hfont)
 	DeleteObject (scaled_font->scaled_hfont);
 
     if (scaled_font->unscaled_hfont)
 	DeleteObject (scaled_font->unscaled_hfont);
 }
 
-static cairo_int_status_t 
+static cairo_int_status_t
 _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
 					 double		x,
 					 double		y,
 					 const char	*utf8,
-					 cairo_glyph_t **glyphs, 
+					 cairo_glyph_t **glyphs,
 					 int		*num_glyphs)
 {
     cairo_win32_scaled_font_t *scaled_font = abstract_font;
@@ -567,7 +569,7 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
     cairo_matrix_transform_distance (&scaled_font->base.font_matrix, &x_incr, &y_incr);
     x_incr /= scaled_font->logical_scale;
     y_incr /= scaled_font->logical_scale;
-    
+
     status = _cairo_utf8_to_utf16 (utf8, -1, &utf16, &n16);
     if (status)
 	return status;
@@ -577,13 +579,13 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
     gcp_results.lpOrder = NULL;
     gcp_results.lpCaretPos = NULL;
     gcp_results.lpClass = NULL;
-    
+
     buffer_size = MAX (n16 * 1.2, 16);		/* Initially guess number of chars plus a few */
     if (buffer_size > INT_MAX) {
 	status = CAIRO_STATUS_NO_MEMORY;
 	goto FAIL1;
     }
-    
+
     hdc = _get_global_font_dc ();
     if (!hdc) {
 	status = CAIRO_STATUS_NO_MEMORY;
@@ -593,7 +595,7 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
     status = cairo_win32_scaled_font_select_font (&scaled_font->base, hdc);
     if (status)
 	goto FAIL1;
-    
+
     while (TRUE) {
 	if (glyph_indices) {
 	    free (glyph_indices);
@@ -603,7 +605,7 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
 	    free (dx);
 	    dx = NULL;
 	}
-	
+
 	glyph_indices = malloc (sizeof (WCHAR) * buffer_size);
 	dx = malloc (sizeof (int) * buffer_size);
 	if (!glyph_indices || !dx) {
@@ -617,7 +619,7 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
 
 	if (!GetCharacterPlacementW (hdc, utf16, n16,
 				     0,
-				     &gcp_results, 
+				     &gcp_results,
 				     GCP_DIACRITIC | GCP_LIGATE | GCP_GLYPHSHAPE | GCP_REORDER)) {
 	    status = _cairo_win32_print_gdi_error ("_cairo_win32_scaled_font_text_to_glyphs");
 	    goto FAIL2;
@@ -627,7 +629,7 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
 	    break;
 
 	/* Too small a buffer, try again */
-	
+
 	buffer_size *= 1.5;
 	if (buffer_size > INT_MAX) {
 	    status = CAIRO_STATUS_NO_MEMORY;
@@ -644,7 +646,7 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
 
     x_pos = x;
     y_pos = y;
-    
+
     for (i = 0; i < gcp_results.nGlyphs; i++) {
 	(*glyphs)[i].index = glyph_indices[i];
 	(*glyphs)[i].x = x_pos ;
@@ -659,16 +661,16 @@ _cairo_win32_scaled_font_text_to_glyphs (void		*abstract_font,
 	free (glyph_indices);
     if (dx)
 	free (dx);
-   
+
     cairo_win32_scaled_font_done_font (&scaled_font->base);
-    
+
  FAIL1:
     free (utf16);
-   
+
     return status;
 }
 
-static cairo_status_t 
+static cairo_status_t
 _cairo_win32_scaled_font_set_metrics (cairo_win32_scaled_font_t *scaled_font)
 {
     cairo_status_t status;
@@ -715,7 +717,7 @@ _cairo_win32_scaled_font_set_metrics (cairo_win32_scaled_font_t *scaled_font)
 	extents.height = (double)(metrics.tmHeight + metrics.tmExternalLeading) / scaled_font->em_square;
 	extents.max_x_advance = (double)(metrics.tmMaxCharWidth) / scaled_font->em_square;
 	extents.max_y_advance = 0;
-	
+
     }
 
     _cairo_scaled_font_set_metrics (&scaled_font->base, &extents);
@@ -723,7 +725,7 @@ _cairo_win32_scaled_font_set_metrics (cairo_win32_scaled_font_t *scaled_font)
     return CAIRO_STATUS_SUCCESS;
 }
 
-static cairo_status_t 
+static cairo_status_t
 _cairo_win32_scaled_font_init_glyph_metrics (cairo_win32_scaled_font_t *scaled_font,
 					     cairo_scaled_glyph_t      *scaled_glyph)
 {
@@ -777,7 +779,7 @@ _cairo_win32_scaled_font_init_glyph_metrics (cairo_win32_scaled_font_t *scaled_f
 	    extents.y_bearing = (- extents.y_bearing - extents.height);
 	    extents.y_advance = - extents.y_advance;
 	}
-	
+
     } else {
 	/* For all other transformations, we use the design metrics
 	 * of the font.
@@ -812,7 +814,7 @@ _cairo_win32_scaled_font_init_glyph_metrics (cairo_win32_scaled_font_t *scaled_f
  * font-space metrics.
  */
 #if 0
-static cairo_status_t 
+static cairo_status_t
 _cairo_win32_scaled_font_glyph_bbox (void		 *abstract_font,
 				     const cairo_glyph_t *glyphs,
 				     int                  num_glyphs,
@@ -867,7 +869,7 @@ _cairo_win32_scaled_font_glyph_bbox (void		 *abstract_font,
 typedef struct {
     cairo_win32_scaled_font_t *scaled_font;
     HDC hdc;
-    
+
     cairo_array_t glyphs;
     cairo_array_t dx;
 
@@ -899,7 +901,7 @@ _flush_glyphs (cairo_glyph_state_t *state)
     status = _cairo_array_append (&state->dx, &dx);
     if (status)
 	return status;
-    
+
     elements = _cairo_array_index (&state->glyphs, 0);
     dx_elements = _cairo_array_index (&state->dx, 0);
     if (!ExtTextOutW (state->hdc,
@@ -911,7 +913,7 @@ _flush_glyphs (cairo_glyph_state_t *state)
 		      dx_elements)) {
 	return _cairo_win32_print_gdi_error ("_flush_glyphs");
     }
-    
+
     _cairo_array_truncate (&state->glyphs, 0);
     _cairo_array_truncate (&state->dx, 0);
 
@@ -937,14 +939,14 @@ _add_glyph (cairo_glyph_state_t *state,
 
     if (state->glyphs.num_elements > 0) {
 	int dx;
-	
+
 	if (logical_y != state->last_y) {
 	    status = _flush_glyphs (state);
 	    if (status)
 		return status;
 	    state->start_x = logical_x;
 	}
-	
+
 	dx = logical_x - state->last_x;
 	status = _cairo_array_append (&state->dx, &dx);
 	if (status)
@@ -955,7 +957,7 @@ _add_glyph (cairo_glyph_state_t *state,
 
     state->last_x = logical_x;
     state->last_y = logical_y;
-    
+
     status = _cairo_array_append (&state->glyphs, &glyph_index);
     if (status)
 	return status;
@@ -995,7 +997,7 @@ _draw_glyphs_on_surface (cairo_win32_surface_t     *surface,
     SetTextColor (surface->dc, color);
     SetTextAlign (surface->dc, TA_BASELINE | TA_LEFT);
     SetBkMode (surface->dc, TRANSPARENT);
-    
+
     _start_glyphs (&state, scaled_font, surface->dc);
 
     for (i = 0; i < num_glyphs; i++) {
@@ -1010,10 +1012,10 @@ _draw_glyphs_on_surface (cairo_win32_surface_t     *surface,
     cairo_win32_scaled_font_done_font (&scaled_font->base);
  FAIL1:
     RestoreDC (surface->dc, -1);
-    
+
     return status;
 }
-		
+
 /* Duplicate the green channel of a 4-channel mask in the alpha channel, then
  * invert the whole mask.
  */
@@ -1066,7 +1068,7 @@ _compute_a8_mask (cairo_win32_surface_t *mask_surface)
     for (i = 0; i < image24->height; i++) {
 	uint32_t *p = (uint32_t *) (image24->data + i * image24->stride);
 	unsigned char *q = (unsigned char *) (image8->data + i * image8->stride);
-	
+
 	for (j = 0; j < image24->width; j++) {
 	    *q = 255 - ((*p & 0x0000ff00) >> 8);
 	    p++;
@@ -1077,7 +1079,6 @@ _compute_a8_mask (cairo_win32_surface_t *mask_surface)
     return &image8->base;
 }
 
-
 static cairo_status_t
 _cairo_win32_scaled_font_glyph_init (void		       *abstract_font,
 				     cairo_scaled_glyph_t      *scaled_glyph,
@@ -1105,7 +1106,7 @@ _cairo_win32_scaled_font_glyph_init (void		       *abstract_font,
     return CAIRO_STATUS_SUCCESS;
 }
 
-static cairo_int_status_t 
+static cairo_int_status_t
 _cairo_win32_scaled_font_show_glyphs (void		       *abstract_font,
 				      cairo_operator_t    	op,
 				      cairo_pattern_t          *pattern,
@@ -1137,7 +1138,7 @@ _cairo_win32_scaled_font_show_glyphs (void		       *abstract_font,
 	 * solid opaque color, we can just call ExtTextOut directly.
 	 */
 	COLORREF new_color;
-	
+
 	new_color = RGB (((int)solid_pattern->color.red_short) >> 8,
 			 ((int)solid_pattern->color.green_short) >> 8,
 			 ((int)solid_pattern->color.blue_short) >> 8);
@@ -1145,10 +1146,10 @@ _cairo_win32_scaled_font_show_glyphs (void		       *abstract_font,
 	status = _draw_glyphs_on_surface (surface, scaled_font, new_color,
 					  0, 0,
 					  glyphs, num_glyphs);
-	
+
 	return status;
     } else {
-	/* Otherwise, we need to draw using software fallbacks. We create a mask 
+	/* Otherwise, we need to draw using software fallbacks. We create a mask
 	 * surface by drawing the the glyphs onto a DIB, black-on-white then
 	 * inverting. GDI outputs gamma-corrected images so inverted black-on-white
 	 * is very different from white-on-black. We favor the more common
@@ -1184,12 +1185,12 @@ _cairo_win32_scaled_font_show_glyphs (void		       *abstract_font,
 		_compute_argb32_mask_alpha (tmp_surface);
 	    else
 		_invert_argb32_mask (tmp_surface);
-	    
+
 	    mask_surface = &tmp_surface->base;
 
 	    /* XXX: Hacky, should expose this in cairo_image_surface */
 	    pixman_image_set_component_alpha (((cairo_image_surface_t *)tmp_surface->image)->pixman_image, TRUE);
-	    
+
 	} else {
 	    mask_surface = _compute_a8_mask (tmp_surface);
 	    cairo_surface_destroy (&tmp_surface->base);
@@ -1203,7 +1204,7 @@ _cairo_win32_scaled_font_show_glyphs (void		       *abstract_font,
 	 */
 	_cairo_pattern_init_for_surface (&mask, mask_surface);
 
-	status = _cairo_surface_composite (op, pattern, 
+	status = _cairo_surface_composite (op, pattern,
 					   &mask.base,
 					   &surface->base,
 					   source_x, source_y,
@@ -1212,7 +1213,7 @@ _cairo_win32_scaled_font_show_glyphs (void		       *abstract_font,
 					   width, height);
 
 	_cairo_pattern_fini (&mask.base);
-	
+
 	cairo_surface_destroy (mask_surface);
 
 	return status;
@@ -1225,7 +1226,7 @@ _cairo_fixed_from_FIXED (FIXED f)
     return *((cairo_fixed_t *)&f);
 }
 
-static cairo_status_t 
+static cairo_status_t
 _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font,
 					  cairo_scaled_glyph_t      *scaled_glyph)
 {
@@ -1248,18 +1249,18 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
     status = cairo_win32_scaled_font_select_font (&scaled_font->base, hdc);
     if (status)
         goto CLEANUP_PATH;
-    
+
     bytesGlyph = GetGlyphOutlineW (hdc, _cairo_scaled_glyph_index (scaled_glyph),
 				   GGO_NATIVE | GGO_GLYPH_INDEX,
 				   &metrics, 0, NULL, &matrix);
-    
+
     if (bytesGlyph == GDI_ERROR) {
 	status = _cairo_win32_print_gdi_error ("_cairo_win32_scaled_font_glyph_path");
 	goto CLEANUP_FONT;
     }
 
     ptr = buffer = malloc (bytesGlyph);
-    
+
     if (!buffer) {
 	status = CAIRO_STATUS_NO_MEMORY;
 	goto CLEANUP_FONT;
@@ -1272,13 +1273,13 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
 	free (buffer);
 	goto CLEANUP_FONT;
     }
-    
+
     while (ptr < buffer + bytesGlyph) {
 	TTPOLYGONHEADER *header = (TTPOLYGONHEADER *)ptr;
 	unsigned char *endPoly = ptr + header->cb;
-	
+
 	ptr += sizeof (TTPOLYGONHEADER);
-	
+
 	_cairo_path_fixed_move_to (path,
 				   _cairo_fixed_from_FIXED (header->pfxStart.x),
 				   _cairo_fixed_from_FIXED (header->pfxStart.y));
@@ -1301,7 +1302,7 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
 		    _cairo_path_fixed_get_current_point (path, &p1x, &p1y);
 		    cx = _cairo_fixed_from_FIXED (points[i].x);
 		    cy = _cairo_fixed_from_FIXED (points[i].y);
-		    
+
 		    if (i + 1 == curve->cpfx - 1) {
 			p2x = _cairo_fixed_from_FIXED (points[i + 1].x);
 			p2y = _cairo_fixed_from_FIXED (points[i + 1].y);
@@ -1311,12 +1312,12 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
 			p2x = (cx + _cairo_fixed_from_FIXED (points[i + 1].x)) / 2;
 			p2y = (cy + _cairo_fixed_from_FIXED (points[i + 1].y)) / 2;
 		    }
-		    
+
 		    c1x = 2 * cx / 3 + p1x / 3;
 		    c1y = 2 * cy / 3 + p1y / 3;
 		    c2x = 2 * cx / 3 + p2x / 3;
 		    c2y = 2 * cy / 3 + p2y / 3;
-		    
+
 		    _cairo_path_fixed_curve_to (path, c1x, c1y, c2x, c2y, p2x, p2y);
 		}
 		break;
@@ -1371,6 +1372,7 @@ typedef struct _cairo_win32_font_face cairo_win32_font_face_t;
 struct _cairo_win32_font_face {
     cairo_font_face_t base;
     LOGFONTW logfont;
+    HFONT hfont;
 };
 
 /* implement the platform-specific interface */
@@ -1390,6 +1392,7 @@ _cairo_win32_font_face_scaled_font_create (void			*abstract_face,
     cairo_win32_font_face_t *font_face = abstract_face;
 
     *font = _win32_scaled_font_create (&font_face->logfont,
+				       font_face->hfont,
 				       &font_face->base,
 				       font_matrix, ctm, options);
     if (*font)
@@ -1409,11 +1412,12 @@ static const cairo_font_face_backend_t _cairo_win32_font_face_backend = {
  * @logfont: A #LOGFONTW structure specifying the font to use.
  *   The lfHeight, lfWidth, lfOrientation and lfEscapement
  *   fields of this structure are ignored.
- * 
+ *
  * Creates a new font for the Win32 font backend based on a
  * #LOGFONT. This font can then be used with
- * cairo_set_font_face() or cairo_font_create(). The #cairo_scaled_font_t
- * returned from cairo_font_create() is also for the Win32 backend
+ * cairo_set_font_face() or cairo_scaled_font_create().
+ * The #cairo_scaled_font_t
+ * returned from cairo_scaled_font_create() is also for the Win32 backend
  * and can be used with functions such as cairo_win32_scaled_font_select_font().
  *
  * Return value: a newly created #cairo_font_face_t. Free with
@@ -1429,9 +1433,42 @@ cairo_win32_font_face_create_for_logfontw (LOGFONTW *logfont)
 	_cairo_error (CAIRO_STATUS_NO_MEMORY);
 	return (cairo_font_face_t *)&_cairo_font_face_nil;
     }
-    
+
     font_face->logfont = *logfont;
-    
+    font_face->hfont = NULL;
+
+    _cairo_font_face_init (&font_face->base, &_cairo_win32_font_face_backend);
+
+    return &font_face->base;
+}
+
+/**
+ * cairo_win32_font_face_create_for_hfont:
+ * @font: An #HFONT structure specifying the font to use.
+ *
+ * Creates a new font for the Win32 font backend based on a
+ * #HFONT. This font can then be used with
+ * cairo_set_font_face() or cairo_scaled_font_create().
+ * The #cairo_scaled_font_t
+ * returned from cairo_scaled_font_create() is also for the Win32 backend
+ * and can be used with functions such as cairo_win32_scaled_font_select_font().
+ *
+ * Return value: a newly created #cairo_font_face_t. Free with
+ *  cairo_font_face_destroy() when you are done using it.
+ **/
+cairo_font_face_t *
+cairo_win32_font_face_create_for_hfont (HFONT font)
+{
+    cairo_win32_font_face_t *font_face;
+
+    font_face = malloc (sizeof (cairo_win32_font_face_t));
+    if (!font_face) {
+	_cairo_error (CAIRO_STATUS_NO_MEMORY);
+	return (cairo_font_face_t *)&_cairo_font_face_nil;
+    }
+
+    font_face->hfont = font;
+
     _cairo_font_face_init (&font_face->base, &_cairo_win32_font_face_backend);
 
     return &font_face->base;
@@ -1456,7 +1493,7 @@ cairo_win32_font_face_create_for_logfontw (LOGFONTW *logfont)
  *
  * Normally, calls to SaveDC() and RestoreDC() would be made around
  * the use of this function to preserve the original graphics state.
- * 
+ *
  * Return value: %CAIRO_STATUS_SUCCESS if the operation succeeded.
  *   otherwise an error such as %CAIRO_STATUS_NO_MEMORY and
  *   the device context is unchanged.
@@ -1503,7 +1540,7 @@ cairo_win32_scaled_font_select_font (cairo_scaled_font_t *scaled_font,
 /**
  * cairo_win32_scaled_font_done_font:
  * @scaled_font: A #cairo_scaled_font_t from the Win32 font backend.
- * 
+ *
  * Releases any resources allocated by cairo_win32_scaled_font_select_font()
  **/
 void
@@ -1514,12 +1551,12 @@ cairo_win32_scaled_font_done_font (cairo_scaled_font_t *scaled_font)
 /**
  * cairo_win32_scaled_font_get_metrics_factor:
  * @scaled_font: a #cairo_scaled_font_t from the Win32 font backend
- * 
+ *
  * Gets a scale factor between logical coordinates in the coordinate
  * space used by cairo_win32_scaled_font_select_font() (that is, the
  * coordinate system used by the Windows functions to return metrics) and
  * font space coordinates.
- * 
+ *
  * Return value: factor to multiply logical units by to get font space
  *               coordinates.
  **/
