@@ -30,6 +30,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <ctype.h>
+#include <setjmp.h>
+#include <signal.h>
 #include <assert.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -67,6 +70,12 @@ typedef enum cairo_internal_surface_type {
 #define access _access
 #define F_OK 0
 #endif
+#ifndef FALSE
+#define FALSE 0
+#endif
+#ifndef TRUE
+#define TRUE !FALSE
+#endif
 
 static void
 xunlink (const char *pathname);
@@ -88,6 +97,12 @@ static const char *fail_face = "", *normal_face = "";
  * general-purpose library, and it keeps the tests cleaner to avoid a
  * context object there, (though not a whole lot). */
 FILE *cairo_test_log_file = NULL;
+char *srcdir;
+
+/* Used to catch crashes in a test, such that we report it as such and
+ * continue testing, although one crasher may already have corrupted memory in
+ * an nonrecoverable fashion. */
+jmp_buf jmpbuf;
 
 void
 cairo_test_init (const char *test_name)
@@ -119,10 +134,10 @@ cairo_test_log (const char *fmt, ...)
 void
 xasprintf (char **strp, const char *fmt, ...)
 {
-#ifdef HAVE_VASPRINTF    
+#ifdef HAVE_VASPRINTF
     va_list va;
     int ret;
-    
+
     va_start (va, fmt);
     ret = vasprintf (strp, fmt, va);
     va_end (va);
@@ -136,7 +151,7 @@ xasprintf (char **strp, const char *fmt, ...)
     va_list va;
     char buffer[BUF_SIZE];
     int ret;
-    
+
     va_start (va, fmt);
     ret = vsnprintf (buffer, sizeof(buffer), fmt, va);
     va_end (va);
@@ -145,12 +160,12 @@ xasprintf (char **strp, const char *fmt, ...)
 	cairo_test_log ("Failure in vsnprintf\n");
 	exit (1);
     }
-    
+
     if (strlen (buffer) == sizeof(buffer) - 1) {
 	cairo_test_log ("Overflowed fixed buffer\n");
 	exit (1);
     }
-    
+
     *strp = strdup (buffer);
     if (!*strp) {
 	cairo_test_log ("Out of memory\n");
@@ -451,7 +466,6 @@ create_glitz_glx_surface (glitz_format_name_t	      formatname,
 	    goto DESTROY_WINDOW;
     }
 
-
     format = glitz_find_standard_format (drawable, formatname);
     if (!format)
 	goto DESTROY_DRAWABLE;
@@ -496,7 +510,7 @@ create_cairo_glitz_glx_surface (cairo_test_t   *test,
 	width = 1;
     if (height == 0)
 	height = 1;
-    
+
     gxtc->dpy = XOpenDisplay (getenv("CAIRO_TEST_GLITZ_DISPLAY"));
     if (!gxtc->dpy) {
 	cairo_test_log ("Failed to open display: %s\n", XDisplayName(0));
@@ -1007,11 +1021,11 @@ create_xcb_surface (cairo_test_t	 *test,
 	cairo_test_log ("Invalid content for XCB test: %d\n", content);
 	return NULL;
     }
-	
+
     render_format = _format_from_cairo (c, format);
     if (render_format.id.xid == 0)
 	return NULL;
-    surface = cairo_xcb_surface_create_with_xrender_format (c, xtc->drawable,
+    surface = cairo_xcb_surface_create_with_xrender_format (c, xtc->drawable, root,
 							    &render_format,
 							    width, height);
 
@@ -1087,7 +1101,7 @@ create_xlib_surface (cairo_test_t	 *test,
 	cairo_test_log ("X server does not have the Render extension.\n");
 	return NULL;
     }
-    
+
     xtc->pixmap = XCreatePixmap (dpy, DefaultRootWindow (dpy),
 				 width, height, xrender_format->depth);
 
@@ -1143,7 +1157,7 @@ create_ps_surface (cairo_test_t		 *test,
 
     /* Sanitize back to a real cairo_content_t value. */
     if (content == CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED)
-	content = CAIRO_CONTENT_COLOR_ALPHA; 
+	content = CAIRO_CONTENT_COLOR_ALPHA;
 
     *closure = ptc = xmalloc (sizeof (ps_target_closure_t));
 
@@ -1159,7 +1173,7 @@ create_ps_surface (cairo_test_t		 *test,
 	free (ptc);
 	return NULL;
     }
-    cairo_ps_surface_set_dpi (surface, 72., 72.);
+    cairo_surface_set_fallback_resolution (surface, 72., 72.);
 
     if (content == CAIRO_CONTENT_COLOR) {
 	ptc->target = surface;
@@ -1214,6 +1228,8 @@ static void
 cleanup_ps (void *closure)
 {
     ps_target_closure_t *ptc = closure;
+    if (ptc->target)
+	cairo_surface_destroy (ptc->target);
     free (ptc->filename);
     free (ptc);
 }
@@ -1222,6 +1238,15 @@ cleanup_ps (void *closure)
 #if CAIRO_HAS_PDF_SURFACE && CAIRO_CAN_TEST_PDF_SURFACE
 #include "cairo-pdf.h"
 
+static const char *pdf_ignored_tests[] = {
+    /* We can't match the results of tests that depend on
+     * CAIRO_ANTIALIAS_NONE, (nor do we care). */
+    "ft-text-antialias-none",
+    "rectangle-rounding-error",
+    "unantialiased-shapes",
+    NULL
+};
+
 cairo_user_data_key_t pdf_closure_key;
 
 typedef struct _pdf_target_closure
@@ -1241,8 +1266,13 @@ create_pdf_surface (cairo_test_t	 *test,
     int height = test->height;
     pdf_target_closure_t *ptc;
     cairo_surface_t *surface;
+    int i;
 
-    /* Sanitizie back to a real cairo_content_t value. */
+    for (i = 0; pdf_ignored_tests[i] != NULL; i++)
+	if (strcmp (test->name, pdf_ignored_tests[i]) == 0)
+	    return NULL;
+
+    /* Sanitize back to a real cairo_content_t value. */
     if (content == CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED)
 	content = CAIRO_CONTENT_COLOR_ALPHA;
 
@@ -1250,7 +1280,7 @@ create_pdf_surface (cairo_test_t	 *test,
 
     ptc->width = width;
     ptc->height = height;
-    
+
     xasprintf (&ptc->filename, "%s-pdf-%s-out.pdf",
 	       test->name, _cairo_test_content_name (content));
 
@@ -1260,7 +1290,7 @@ create_pdf_surface (cairo_test_t	 *test,
 	free (ptc);
 	return NULL;
     }
-    cairo_pdf_surface_set_dpi (surface, 72., 72.);
+    cairo_surface_set_fallback_resolution (surface, 72., 72.);
 
     if (content == CAIRO_CONTENT_COLOR) {
 	ptc->target = surface;
@@ -1317,6 +1347,8 @@ static void
 cleanup_pdf (void *closure)
 {
     pdf_target_closure_t *ptc = closure;
+    if (ptc->target)
+	cairo_surface_destroy (ptc->target);
     free (ptc->filename);
     free (ptc);
 }
@@ -1326,9 +1358,10 @@ cleanup_pdf (void *closure)
 #include "cairo-svg.h"
 
 static const char *svg_ignored_tests[] = {
-    /* rectangle-rounding-error uses CAIRO_ANTIALIAS_NONE, 
+    /* rectangle-rounding-error uses CAIRO_ANTIALIAS_NONE,
      * which is not supported */
-    "rectangle-rounding-error",		
+    "ft-text-antialias-none",
+    "rectangle-rounding-error",
     NULL
 };
 
@@ -1353,28 +1386,28 @@ create_svg_surface (cairo_test_t	 *test,
     cairo_surface_t *surface;
 
     for (i = 0; svg_ignored_tests[i] != NULL; i++)
-	if (strcmp (test->name, svg_ignored_tests[i]) == 0)	   
+	if (strcmp (test->name, svg_ignored_tests[i]) == 0)
 	    return NULL;
 
     *closure = ptc = xmalloc (sizeof (svg_target_closure_t));
 
     ptc->width = width;
     ptc->height = height;
-    
+
     xasprintf (&ptc->filename, "%s-svg-%s-out.svg",
 	       test->name, _cairo_test_content_name (content));
-    
+
     surface = cairo_svg_surface_create (ptc->filename, width, height);
     if (cairo_surface_status (surface)) {
 	free (ptc->filename);
 	free (ptc);
 	return NULL;
     }
-    cairo_svg_surface_set_dpi (surface, 72., 72.);
+    cairo_surface_set_fallback_resolution (surface, 72., 72.);
 
     if (content == CAIRO_CONTENT_COLOR) {
 	ptc->target = surface;
-	surface = cairo_surface_create_similar (ptc->target, 
+	surface = cairo_surface_create_similar (ptc->target,
 						CAIRO_CONTENT_COLOR,
 						width, height);
     } else {
@@ -1419,7 +1452,7 @@ svg_surface_write_to_png (cairo_surface_t *surface, const char *filename)
 
     if (system (command) != 0)
 	return CAIRO_STATUS_WRITE_ERROR;
-    
+
     return CAIRO_STATUS_SUCCESS;
 }
 
@@ -1427,6 +1460,8 @@ static void
 cleanup_svg (void *closure)
 {
     svg_target_closure_t *ptc = closure;
+    if (ptc->target)
+	cairo_surface_destroy (ptc->target);
     free (ptc->filename);
     free (ptc);
 }
@@ -1442,21 +1477,13 @@ cairo_test_for_target (cairo_test_t *test,
     cairo_surface_t *surface;
     cairo_t *cr;
     char *png_name, *ref_name, *diff_name, *offset_str;
-    char *srcdir;
     char *format;
     cairo_test_status_t ret;
+    cairo_content_t expected_content;
 
     /* Get the strings ready that we'll need. */
-    srcdir = getenv ("srcdir");
-    if (!srcdir)
-	srcdir = ".";
     format = _cairo_test_content_name (target->content);
 
-    if (dev_offset)
-	xasprintf (&offset_str, "-%d", dev_offset);
-    else
-	offset_str = strdup("");
-    
     if (dev_offset)
 	xasprintf (&offset_str, "-%d", dev_offset);
     else
@@ -1505,6 +1532,7 @@ cairo_test_for_target (cairo_test_t *test,
 	goto UNWIND_STRINGS;
     }
 
+    /* Check that we created a surface of the expected type. */
     if (cairo_surface_get_type (surface) != target->expected_type) {
 	cairo_test_log ("Error: Created surface is of type %d (expected %d)\n",
 			cairo_surface_get_type (surface), target->expected_type);
@@ -1512,6 +1540,21 @@ cairo_test_for_target (cairo_test_t *test,
 	goto UNWIND_SURFACE;
     }
 
+    /* Check that we created a surface of the expected content,
+     * (ignore the articifical
+     * CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED value).
+     */
+    expected_content = target->content;
+    if (expected_content == CAIRO_TEST_CONTENT_COLOR_ALPHA_FLATTENED)
+	expected_content = CAIRO_CONTENT_COLOR_ALPHA;
+
+    if (cairo_surface_get_content (surface) != expected_content) {
+	cairo_test_log ("Error: Created surface has content %d (expected %d)\n",
+			cairo_surface_get_content (surface), expected_content);
+	ret = CAIRO_TEST_FAILURE;
+	goto UNWIND_SURFACE;
+    }
+
     cairo_surface_set_device_offset (surface, dev_offset, dev_offset);
 
     cr = cairo_create (surface);
@@ -1580,15 +1623,22 @@ UNWIND_STRINGS:
     return ret;
 }
 
+static void
+segfault_handler (int signal)
+{
+    longjmp (jmpbuf, signal);
+}
+
 static cairo_test_status_t
 cairo_test_expecting (cairo_test_t *test, cairo_test_draw_function_t draw,
 		      cairo_test_status_t expectation)
 {
-    int i, j, num_targets;
+    volatile int i, j, num_targets;
     const char *tname;
+    sighandler_t old_segfault_handler;
     cairo_test_status_t status, ret;
     cairo_test_target_t **targets_to_test;
-    cairo_test_target_t targets[] = 
+    cairo_test_target_t targets[] =
 	{
 	    { "image", CAIRO_SURFACE_TYPE_IMAGE, CAIRO_CONTENT_COLOR_ALPHA,
 	      create_image_surface, cairo_surface_write_to_png, NULL},
@@ -1622,26 +1672,26 @@ cairo_test_expecting (cairo_test_t *test, cairo_test_draw_function_t draw,
 #if CAIRO_CAN_TEST_GLITZ_GLX_SURFACE
 	    { "glitz-glx", CAIRO_SURFACE_TYPE_GLITZ,CAIRO_CONTENT_COLOR_ALPHA,
 		create_cairo_glitz_glx_surface, cairo_surface_write_to_png,
-		cleanup_cairo_glitz_glx }, 
+		cleanup_cairo_glitz_glx },
 	    { "glitz-glx", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR,
 		create_cairo_glitz_glx_surface, cairo_surface_write_to_png,
-		cleanup_cairo_glitz_glx }, 
+		cleanup_cairo_glitz_glx },
 #endif
 #if CAIRO_CAN_TEST_GLITZ_AGL_SURFACE
 	    { "glitz-agl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR_ALPHA,
 		create_cairo_glitz_agl_surface, cairo_surface_write_to_png,
-		cleanup_cairo_glitz_agl }, 
+		cleanup_cairo_glitz_agl },
 	    { "glitz-agl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR,
 		create_cairo_glitz_agl_surface, cairo_surface_write_to_png,
-		cleanup_cairo_glitz_agl }, 
+		cleanup_cairo_glitz_agl },
 #endif
 #if CAIRO_CAN_TEST_GLITZ_WGL_SURFACE
 	    { "glitz-wgl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR_ALPHA,
 		create_cairo_glitz_wgl_surface, cairo_surface_write_to_png,
-		cleanup_cairo_glitz_wgl }, 
+		cleanup_cairo_glitz_wgl },
 	    { "glitz-wgl", CAIRO_SURFACE_TYPE_GLITZ, CAIRO_CONTENT_COLOR,
 		create_cairo_glitz_wgl_surface, cairo_surface_write_to_png,
-		cleanup_cairo_glitz_wgl }, 
+		cleanup_cairo_glitz_wgl },
 #endif
 #endif /* CAIRO_HAS_GLITZ_SURFACE */
 #if 0 && CAIRO_HAS_QUARTZ_SURFACE
@@ -1718,21 +1768,46 @@ cairo_test_expecting (cairo_test_t *test, cairo_test_draw_function_t draw,
 #endif
 	};
 
+#ifdef HAVE_UNISTD_H
+    if (isatty (1)) {
+	fail_face = "\033[41m\033[37m\033[1m";
+	normal_face = "\033[m";
+    }
+#endif
+
+    srcdir = getenv ("srcdir");
+    if (!srcdir)
+	srcdir = ".";
+
     if ((tname = getenv ("CAIRO_TEST_TARGET")) != NULL) {
 	const char *tname = getenv ("CAIRO_TEST_TARGET");
 	num_targets = 0;
 	targets_to_test = NULL;
-	/* realloc isn't exactly the best thing here, but meh. */
-	for (i = 0; i < sizeof(targets)/sizeof(targets[0]); i++) {
-	    if (strcmp (targets[i].name, tname) == 0) {
-		targets_to_test = realloc (targets_to_test, sizeof(cairo_test_target_t *) * (num_targets+1));
-		targets_to_test[num_targets++] = &targets[i];
-	    }
-	}
 
-	if (num_targets == 0) {
-	    fprintf (stderr, "CAIRO_TEST_TARGET '%s' not found in targets list!\n", tname);
-	    exit(-1);
+	while (*tname) {
+	    int found = 0;
+	    const char *end = strpbrk (tname, " \t;:,");
+	    if (!end)
+	        end = tname + strlen (tname);
+
+	    for (i = 0; i < sizeof(targets)/sizeof(targets[0]); i++) {
+		if (strncmp (targets[i].name, tname, end - tname) == 0 &&
+		    !isalnum (targets[i].name[end - tname])) {
+		    /* realloc isn't exactly the best thing here, but meh. */
+		    targets_to_test = realloc (targets_to_test, sizeof(cairo_test_target_t *) * (num_targets+1));
+		    targets_to_test[num_targets++] = &targets[i];
+		    found = 1;
+		}
+	    }
+
+	    if (!found) {
+		fprintf (stderr, "CAIRO_TEST_TARGET '%s' not found in targets list!\n", tname);
+		exit(-1);
+	    }
+
+	    if (*end)
+	      end++;
+	    tname = end;
 	}
     } else {
 	num_targets = sizeof(targets)/sizeof(targets[0]);
@@ -1747,7 +1822,7 @@ cairo_test_expecting (cairo_test_t *test, cairo_test_draw_function_t draw,
      * iff. there is at least one tested backend and that all tested
      * backends return SUCCESS. In other words:
      *
-     *	if      any backend FAILURE
+     *	if      any backend not SUCCESS
      *		-> FAILURE
      *	else if all backends UNTESTED
      *		-> FAILURE
@@ -1765,7 +1840,13 @@ cairo_test_expecting (cairo_test_t *test, cairo_test_draw_function_t draw,
 		    _cairo_test_content_name (target->content),
 		    dev_offset);
 
-	    status = cairo_test_for_target (test, draw, target, dev_offset);
+	    /* Set up a checkpoint to get back to in case of segfaults. */
+	    old_segfault_handler = signal (SIGSEGV, (sighandler_t) segfault_handler);
+	    if (0 == setjmp (jmpbuf))
+		status = cairo_test_for_target (test, draw, target, dev_offset);
+	    else
+	        status = CAIRO_TEST_CRASHED;
+	    signal (SIGSEGV, (sighandler_t) old_segfault_handler);
 
 	    cairo_test_log ("TEST: %s TARGET: %s FORMAT: %s OFFSET: %d RESULT: ",
 			    test->name, target->name,
@@ -1783,6 +1864,11 @@ cairo_test_expecting (cairo_test_t *test, cairo_test_draw_function_t draw,
 		printf ("UNTESTED\n");
 		cairo_test_log ("UNTESTED\n");
 		break;
+	    case CAIRO_TEST_CRASHED:
+		printf ("%s!!!CRASHED!!!%s\n", fail_face, normal_face);
+		cairo_test_log ("CRASHED\n");
+		ret = CAIRO_TEST_FAILURE;
+		break;
 	    default:
 	    case CAIRO_TEST_FAILURE:
 		if (expectation == CAIRO_TEST_FAILURE) {
@@ -1812,7 +1898,7 @@ cairo_test_expecting (cairo_test_t *test, cairo_test_draw_function_t draw,
 }
 
 cairo_test_status_t
-cairo_test_expect_failure (cairo_test_t		      *test, 
+cairo_test_expect_failure (cairo_test_t		      *test,
 			   cairo_test_draw_function_t  draw,
 			   const char		      *because)
 {
@@ -1823,12 +1909,6 @@ cairo_test_expect_failure (cairo_test_t		      *test,
 cairo_test_status_t
 cairo_test (cairo_test_t *test, cairo_test_draw_function_t draw)
 {
-#ifdef HAVE_UNISTD_H
-    if (isatty (1)) {
-	fail_face = "\033[41m\033[37m\033[1m";
-	normal_face = "\033[m";
-    }
-#endif
     printf ("\n");
     return cairo_test_expecting (test, draw, CAIRO_TEST_SUCCESS);
 }
@@ -1840,8 +1920,8 @@ cairo_test_create_surface_from_png (const char *filename)
     char *srcdir = getenv ("srcdir");
 
     image = cairo_image_surface_create_from_png (filename);
-    if (cairo_surface_status(image)) { 
-        /* expect not found when running with srcdir != builddir 
+    if (cairo_surface_status(image)) {
+        /* expect not found when running with srcdir != builddir
          * such as when 'make distcheck' is run
          */
 	if (srcdir) {
